; VIDEO.ASM
;
; Routines for VGA video I/O.
;

.model	large, c

.data

vsegs	dw	seg    x720y540, seg    x360y480
voffs	dw	offset x720y540, offset x360y480

x720y540 db	720/8
	db	540/16
	db	6Ah, 59h, 5Ah, 8Dh
	db	5Eh, 8Bh, 4AH,0F0h
	db	 0h, 60h,  0h,	0h
	db	 0h,  0h,  2h, 49h
	db	24h, 86h, 1Bh, 2Dh
	db	 0h, 24h, 44h,0E3h
	db	0FFh

x360y480 db	360/8
	db	480/16
	db	 6bh, 59h, 5ah, 8eh
	db	 5eh, 8ah, 0DH,03Eh
	db	  0h, 40h, 00h,  0h
	db	  0h,  0h,  0h, 31h
	db	0EAh, 0ACh, 0DFh, 2Dh
	db	  0h,0E7h, 06h,0E3h
	db	0FFh

.code

;
;	int setmode(int mode);
;

setmode	proc	uses ds si, mode:word

mov	bx, mode
	cmp	bx, 20h
	jl	biosmode
	cmp	bx, 21h
	jle	goodmode
badmode:
	mov	ax, -1
	jmp	smout
biosmode:
	mov	ax, bx
	int	10h
	jmp	smout1
goodmode:
	sub	bx, 20h
	shl	bx, 1
	mov	ax, word ptr [vsegs+bx]
	mov	ds, ax
	mov	si, word ptr [voffs+bx]

	mov	ax, 0012h		; Clear video RAM
	int	10h

        cmp	bx, 0
	je	not256

	mov	ax, 0013h
	int	10h

	mov	dx, 3c4h       		; Alter sequencer registers
	mov	ax, 0604h		; Disable chain 4
	out	dx, ax

	jmp	short is256		; Forget the ROM characters
not256:
	mov	ax, 1124h		; Load ROM 8*16 characters
	mov	bx, 0
	mov	dh, 0
	mov	dl, byte ptr [si+1]	; Number of rows on the screen
	int	10h
is256:
	mov	ax, 40h			; Video BIOS data area
	mov	es, ax

	mov	ah, 0
	mov	al, byte ptr [si]	; Number of columns on the screen
	mov	word ptr es:[4ah], ax
	mul	byte ptr [si+1]		; Number of characters on the screen
	shl	ax, 1			; (attributes, also)
	mov	word ptr es:[4ch], ax

	mov	dx, word ptr es:[63h]	; Where's the 6845?
	add	dx, 6			; Locate the status register
vrdly1:
	in	al, dx			; Loop until vertical retrace is off
	test	al, 8
	jnz	vrdly1
vrdly2:
	in	al, dx			; Now loop until it's on
	test	al, 8
	jz	vrdly2

	cli
	mov	dx, 03c4h		; Sequencer Synchronous reset
	mov	ax, 0100h		;	asserted
	out	dx, ax
	mov	dx, 03c2h		; Update Misc Output Reg
	mov	al, 0E7h
	out	dx, al
	mov	dx, 03c4h		; Sequencer Synchronous reset
	mov	ax, 0300h		; 	cleared
	out	dx, ax

	mov	dx, word ptr es:[63h]
	add	si, 2
	mov	al, 11h			; Deprotect registers 0-7
	mov	ah, byte ptr [si+11h]
	and	ah, 7fh
	out	dx, ax

	mov	cx, 18h
	mov	bx, 00
crtcloop:
	mov	al, bl
	mov	ah, byte ptr [bx+si]
	out	dx, ax
	inc	bx
	loop	crtcloop
	sti
smout1:
	mov	ax, 0			; All's well
smout:
	ret

setmode	endp

;
;	void putdot13(U8 color, int x, int y);
;

putdot13 proc	uses es di, color:word, x:word, y:word

	mov	bx, color
	mov	cx, x
	mov	dx, y

	mov	ax, 0a000h
	mov	es, ax

	mov	ax, 320
	mul	dx
	add	ax, cx
	mov	di, ax
	mov	es:[di], bl

	ret

putdot13 endp

;
;	void putdot21(U8 color, int x, int y);
;

putdot21 proc	uses es di, color:word, x:word, y:word

	mov	bx, color
	mov	cx, x
	mov	dx, y

	mov	ax, 0a000h
	mov	es, ax

	mov	ax, 90
	mul	dx
	push	cx
	shr	cx, 1
	shr	cx, 1
	add	ax, cx
	mov	di, ax
	pop	cx
	and	cl, 3
	mov	ah, 1
	shl	ah, cl

	mov	al, 2
	mov	dx, 3c4h
	out     dx, ax
	mov	es:[di], bl

	ret

putdot21 endp

;
;
;

readdac	proc	uses es, array:ptr byte

	les	dx, array
	mov	ax, 1017h
	mov	bx, 0
	mov	cx, 256
	int	10h
	ret

readdac	endp

writedac proc	uses es, array:ptr byte

	les	dx, array
	mov	ax, 1012h
	mov	bx, 0
	mov	cx, 256
	int	10h
	ret

writedac endp

	end
